local WAND_RANGE, PLAYER_HEIGHT, p3, origin, posplus, posm, round, roundpos, one_to_left, one_to_right, one_up, one_down, adjust_pos
WAND_RANGE = 20
PLAYER_HEIGHT = 1 + 10 / 16
p3 = function(x, y, z)
  return {
    x = x,
    y = y,
    z = z
  }
end
origin = p3(0, 0, 0)
posplus = function(pos1, pos2)
  return {
    x = pos1.x + pos2.x,
    y = pos1.y + pos2.y,
    z = pos1.z + pos2.z
  }
end
posm = function(scale, pos)
  return {
    x = scale * pos.x,
    y = scale * pos.y,
    z = scale * pos.z
  }
end
round = function(x)
  return math.floor(x + .5)
end
roundpos = function(pos)
  return {
    x = round(pos.x),
    y = round(pos.y),
    z = round(pos.z)
  }
end
one_to_left = function(v)
  return p3((function()
    return -v.z / math.sqrt(v.z ^ 2 + v.x ^ 2)
  end)(), v.y, v.x / math.sqrt(v.z ^ 2 + v.x ^ 2))
end
one_to_right = function(v)
  return p3((function()
    return v.z / math.sqrt(v.z ^ 2 + v.x ^ 2)
  end)(), v.y, -v.x / math.sqrt(v.z ^ 2 + v.x ^ 2))
end
one_up = function(v)
  local l1 = math.sqrt(v.z ^ 2 + v.x ^ 2)
  local l2 = math.sqrt(v.z ^ 2 + v.y ^ 2 + v.x ^ 2)
  return {
    x = -v.x * v.y / (l1 * l2),
    y = l1 / l2,
    z = -v.y * v.z / (l1 * l2)
  }
end
one_down = function(v)
  return posm(-1, one_up(v))
end
adjust_pos = function(pos, dir, h, v)
  if h > 0 then
    pos = posplus(pos, posm(h, one_to_left(dir)))
  end
  if h < 0 then
    pos = posplus(pos, posm(math.abs(h), one_to_right(dir)))
  end
  if v > 0 then
    pos = posplus(pos, posm(v, one_up(dir)))
  end
  if v < 0 then
    pos = posplus(pos, posm(math.abs(v), one_down(dir)))
  end
  return pos
end
return minetest.register_craftitem('polaris:wand_digging', {
  description = 'Wand of Digging',
  inventory_image = 'polaris_wand_digging.png',
  on_use = function(itemstack, user, pointed_thing)
    local dir = posm(.5, user:get_look_dir())
    local pos = posplus(user:getpos(), p3(0, PLAYER_HEIGHT, 0))
    pos = posplus(pos, posm(2, dir))
    local rays
    do
      local _accum_0 = { }
      local _len_0 = 1
      for h = -1, 1 do
        for v = -1, 1 do
          _accum_0[_len_0] = adjust_pos(pos, dir, h, v)
          _len_0 = _len_0 + 1
        end
      end
      rays = _accum_0
    end
    for n = 1, 2 * WAND_RANGE do
      for i, r in ipairs(rays) do
        minetest.remove_node(roundpos(r))
        rays[i] = posplus(r, dir)
      end
    end
    itemstack:take_item()
    return itemstack
  end
})
